#!/usr/bin/env python

"""
The base, abstract, class that defines the methods for updating
and getting specific information from the database.
"""

__cvsversion__ = '$Id: DB.py,v 1.10 2007/12/06 23:32:20 jay Exp $'
__author__ = 'Jay Deiman; jay@splitstreams.com'
__copyright__ = 'Copyright 2007 Jason Deiman'
__license__ = 'GPLv3'
__all__ = ['DB']

import threading , time , utils

class DB(threading.Thread):
    """
    Base class which defines the methods to be overridden by the
    different actual DB implementations.
    """
    def __init__(self , config , actionQ , fwQ , reqQ , respQ , logger):
        """
        Initialize a couple of vars and init the parent
        """
        self._lastCleanup = 0
        self._cleanupInterval = 60      # 60 seconds
        self._accessExpire = 2592000    # 30 days
        self._config = config
        self._actionQ = actionQ
        self._fwQ = fwQ
        self._reqQ = reqQ
        self._respQ = respQ
        self._logger = logger
        self._conn = None
        self._cursor = None
        threading.Thread.__init__(self)
        
    def _cleanup(self):
        """
        Runs the database cleanup routine, removing old entries,
        expiring bans, etc.
        """
        raise NotImplementedError , 'You must override this method'
    
    def _initDB(self):
        """
        Initialize the database if it does not exist
        """
        raise NotImplementedError , 'You must override this method'
    
    def run(self):
        """
        The main program loop.  This reads from the actionQ and runs
        the add() method as well as the cleanup
        """
        self._conn = self._initDB()
        self._cursor = self._conn.cursor()
        while True:
            time.sleep(0.1)
            # Try the access queue and see if there are new items on it
            try:
                ip , logLine = self._actionQ.get(block=False)
            except:
                pass
            else:
                self.add(ip , logLine)
                if utils.DEBUG:
                    print 'Adding access for %s' % ip
            # Try the request queue and see if there are new items on it
            try:
                req = self._reqQ.get(block=False)
            except:
                pass
            else:
                data = []
                if type(req) == type(('',)):
                    exec('self.%s("%s")' % (req[0] , req[1]))
                else:
                    exec('data = self.%s()' % req)
                    self._respQ.put(data)
            if not self._lastCleanup or \
                    time.time() > self._lastCleanup + self._cleanupInterval:
                self._lastCleanup = time.time()
                self._cleanup()
    
    def add(self , ip ,  logLine):
        raise NotImplementedError , 'You must override this method'
    
    def isBanned(self , ip):
        raise NotImplementedError , 'You must override this method'
    
    def ban(self , ip):
        raise NotImplementedError , 'You must override this method'
    
    def unban(self , ip):
        raise NotImplementedError , 'You must override this method'
    
    def banPermanent(self , ip):
        raise NotImplementedError , 'You must override this method'
    
    def getBanned(self):
        raise NotImplementedError , 'You must override this method'
    
    def getAccesses(self):
        raise NotImplementedError , 'You must override this method'
    
    def getPrevBanned(self):
        raise NotImplementedError , 'You must override this method'