"""
The class which defines the manipulation of null routes
"""

__cvsversion__ = '$Id: NullRtRules.py,v 1.7 2007/12/06 23:32:20 jay Exp $'
__author__ = 'Jay Deiman; jay@splitstreams.com'
__copyright__ = 'Copyright 2007 Jason Deiman'
__license__ = 'GPLv3'
__all__ = ['NullRtRules']

from subprocess import *
import os , sys , FWRules , utils

class NullRtRules(FWRules.FWRules):
    """
    Wrapper for the system calls to 'route' for the blocking and
    unblocking of IPs
    """
    def __init__(self , config , fwQ , logger):
        self._banTemplate = ''
        self._unbanTemplate = ''
        if sys.platform[:5] == 'linux':
            self._banTemplate = '%s add %s gw 127.0.0.1'
            self._unbanTemplate = '%s del %s gw 127.0.0.1'
        else:
            # Default to BSD style route command
            self._banTemplate = '%s add %s 127.0.0.1'
            self._unbanTemplate = '%s delete %s 127.0.0.1'
        FWRules.FWRules.__init__(self , config , fwQ , logger)
        
    def ban(self , ip):
        cmd = self._banTemplate % (self._config.RuleCmdPath , ip)
        if utils.DEBUG:
            print cmd
        
        proc = Popen(cmd , shell=True , bufsize=4096 , stdin=PIPE , 
                     stderr=STDOUT , stdout=PIPE , close_fds=True)
        proc.stdin.close()
        output = proc.stdout.read()
        proc.stdout.close()
        retCode = int(proc.wait())
        if retCode != 0:
            self._logger.log(output , self._logger.LOG_ERR)
        else:
            self._logger.log('Null routed %s' % ip)
            
    def unban(self , ip):
        cmd = self._unbanTemplate % (self._config.RuleCmdPath , ip)
        if utils.DEBUG:
            print cmd
        
        proc = Popen(cmd , shell=True , bufsize=4096 , stdin=PIPE , 
                 stderr=STDOUT , stdout=PIPE , close_fds=True)
        proc.stdin.close()
        output = proc.stdout.read()
        proc.stdout.close()
        retCode = int(proc.wait())
        if retCode != 0:
            self._logger.log(output , self._logger.LOG_ERR)
        else:
            self._logger.log('Removed null route for %s' % ip)